<?php
/**
 * User Logic
 * @package     User
 * @copyright   Copyright (C) 2011 MicroEngine Inc.
 */
class User_Logic
{
    /**
     * DAOオブジェクト
     * @var User_Dao
     */
    private $dao;

    /**
     * コンストラクタ
     */
    public function __construct()
    {
        $this->dao = new User_Dao();
    }

    /**
     * ユーザー一覧
     * @return array $rows
     */
    public function select_list()
    {
        return $this->dao->select_list();
    }

    /**
     * ユーザー取得
     * @param string $_id _idフィールド値
     * @param string $class_name クラス名
     * @return object $obj
     */
    public function select_user($_id, $class_name = null)
    {
        return $this->dao->select($_id, $class_name);
    }

    /**
     * ユーザー登録
     * @param User_Form $form
     * @param boolean $install_flag
     * @return boolean
     */
    public function register_user(User_Form $form, $install_flag = false)
    {
        // admin以外のばあい
        if (!$install_flag && !User_Role::require_admin(false)) {
            // 編集の場合は、ユーザー名、ロールは更新させない
            if (!empty($form->_id)) {
                $user = $this->dao->select($form->_id);
                $form->user_name = $user->user_name;
                $form->user_role = $user->user_role;
                $form->default_access = $user->default_access;
                $form->path_config = $user->path_config;
            } else {
                throw new Me_Exception(608);
            }
        }

        if (!$this->validation($form)) {
            return false;
        }

        if (empty($form->_id)) {
            // 新規追加
            $this->dao->insert_user($form);
        } else {
            // 編集
            $this->dao->update_user($form);
            // ログインユーザーの名前を変更した場合は、セッション内の情報も更新する。
            $login_user = $_SESSION['login_user'];
            if ($form->_id === $login_user->_id) {
                $login_user->user_name = $form->user_name;
                $login_user->display_name = $form->display_name;
            }
        }
        return true;
    }

    /**
     * ユーザー削除
     * @param object $user
     */
    public function delete_user($user)
    {
        if (!$this->dao->delete($user)) {
            throw new Me_Exception(600);
        }
    }

    /**
     * 編集可能判定
     * @param string $user_id
     * @return boolean $result
     */
    public function is_editable($user_id)
    {
        $result = true;
        // システム管理者以外は、自分自身しか編集できない。
        if (!User_Role::require_admin(false)) {
            $login_user = $_SESSION['login_user'];
            if ($login_user->_id !== $user_id) {
                // 自分自身は以外は編集できない。
                $result = false;
            }
        }
        return $result;
    }

    /**
     * 削除可能判定
     * @param object $user
     * @return boolean $result
     */
    public function is_deletable($user)
    {
        $result = true;
        $login_user = $_SESSION['login_user'];
        if ($login_user->_id === $user->_id) {
            // 自分自身は削除できない。
            $result = false;
        }
        return $result;
    }

    /**
     * 認証処理
     * @return array (User $login_user)
     */
    public function authentication()
    {
        // セッションを開始
        session_name(Me_Config::get('session.name'));
        session_set_cookie_params(0, CMS_PATH . '/');
        session_start();

        // セッションにuserオブジェクトが存在するか
        if (isset($_SESSION['login_user'])
            && $_SESSION['login_user'] instanceof User
            && $_SESSION['me_session_id'] === Me_Config::get('me_session_id')
        ) {
            // セッションからuserオブジェクトを取得
            $login_user = $_SESSION['login_user'];
            return $login_user;
        } else {
            if (isset($_POST['user_name'])) {
                // ログイン画面からPOSTされた場合
                $form = new User_Form();

                //POSTされたデータをセットする。
                $form->user_name = $_POST['user_name'];
                $form->password = $_POST['password'];

                // 必須チェック
                if (empty($form->user_name)) {
                    $form->errors['user_name'] = 'ユーザー名を入力して下さい';
                }
                if (empty($form->password)) {
                    $form->errors['password'] = 'パスワードを入力して下さい';
                }
                // エラー有無確認
                if (!empty($form->errors)) {
                    $form->errors['MESSAGE'] = 'エラーが発生しました。';
                    // ログイン画面を表示
                    $this->render_login_page($form);
                }

                // 認証処理
                $result = null;
                $login_user = null;
                list($result, $login_user) = $this->dao->select_auth($form);
                if ($result) {
                    // ログイン成功
                    // セッション情報を破棄
                    $_SESSION = array();
                    session_destroy();
                    session_start();

                    // セッションIDを再生成する
                    session_regenerate_id(true);
                    // 認証成功
                    $_SESSION['login_user'] = $login_user;
                    $_SESSION['me_session_id'] = Me_Config::get('me_session_id');
                    // ライセンス認証
                    License_Util::auth();
                    // flashメッセージに試用期間情報を渡す。
                    $_SESSION['_flash'] = License_Util::get_message();

                    // ログ書き出し
                    $log_text = array(
                        $login_user->user_name,
                        $login_user->display_name,
                        User_Role::get_role_name($login_user->user_role),
                        $_SERVER['HTTP_USER_AGENT']
                    );
                    Me_Logger::log(1000, $log_text);
                    // refreshする
                    Me_Util::refresh();
                } else {
                    $form->errors['MESSAGE'] = '認証に失敗しました。ユーザー名または、'
                        . 'パスワードが正しくありません。';
                    // ログ書き出し
                    $log_text = array(
                        $form->user_name,
                        $_SERVER['HTTP_USER_AGENT']
                    );
                    Me_Logger::log(1001, $log_text);

                    // ログイン画面を表示
                    $this->render_login_page($form);
                }
            } else {
                // ログイン画面を表示
                $this->render_login_page();
            }
        }
    }

    /**
     * ログイン画面出力
     * @param User_Form $form
     */
    private function render_login_page($form = null)
    {
        if (is_null($form)) {
            $form = new User_Form();
        }
        // ログイン画面を表示
        h::render('/Common/login.php', compact('form'));
        exit;
    }

    /**
     * 入力値チェック
     * @param User_Form $form
     * @return boolean
     */
    public function validation(User_Form $form)
    {
        // エラーメッセージを初期化
        $form->errors = array();

        // ユーザー名
        // 最小文字数チェック
        if (strlen($form->user_name) < 1) {
            $form->errors['user_name'] = 'ユーザー名を入力して下さい';
        } else if (strlen($form->user_name) > 32) {
            // 最大文字数チェック
            $form->errors['user_name'] = 'ユーザー名は32文字以内で入力して下さい';
        } else if (!preg_match('/^[a-z0-9@._-]+$/i', $form->user_name)) {
            // 文字種チェック
            $form->errors['user_name'] = 'ユーザー名は、英字、数字、ハイフン( - ),' .
                ' アンダースコア( _ ), ピリオド( . ), アットマーク( @ )のみで入力して下さい';
        } else {
            // 既存ユーザーとの重複確認
            $exist = $this->dao->exist_user($form->user_name);
            if (empty($form->_id)) {
                // 新規
                if ($exist) {
                    $form->errors['user_name'] = '指定されたユーザー名は既に存在します。';
                }
            } else {
                // 更新時は、変更が無いか確認
                $old_user = $this->dao->select($form->_id);
                if ($exist && $form->user_name !== $old_user->user_name) {
                    $form->errors['user_name'] = '指定されたユーザー名は既に存在します。';
                }
            }
        }

        // パスワードチェック
        // 初期設定時またはパスワードが入力されているときはチェックする。
        if (empty($form->_id)
            || !empty($form->password)
            || !empty($form->confirm_password)
        ) {
            //最小文字数チェック
            if (strlen($form->password) < 4) {
                $form->errors['password'] = 'パスワードは4文字以上で入力して下さい';
            } else if (strlen($form->password) > 32) {
                //最大文字数チェック
                $form->errors['password'] = 'パスワードは32文字以内で入力して下さい';
            } else if ($form->password != $form->confirm_password) {
                //確認用パスワードとの一致チェック
                $form->errors['password'] = 'パスワードが確認用パスワードと異なります';
            } else if (!preg_match('/^[a-z0-9@._-]+$/i', $form->password)) {
                //文字種チェック
                $form->errors['password'] = 'パスワードは、英字、数字、ハイフン( - ),' .
                    ' アンダースコア( _ ), ピリオド( . ), アットマーク( @ )のみで入力して下さい';
            }
        }

        // 表示名チェック
        if (mb_strlen($form->display_name) > 32) {
            $form->errors['display_name'] = '表示名は32文字以内で入力してください';
        }

        // 備考チェック
        if (mb_strlen($form->note) > 50) {
            $form->errors['note'] = '備考は50文字以内で入力してください';
        }

        // エラー有無確認
        if (!empty($form->errors)) {
            $form->errors['MESSAGE'] = 'エラーが発生しました。';
            return false;
        }
        return true;
    }

    /**
     * ログアウト
     */
    public function logout()
    {
        // ログインユーザー情報取得
        $login_user = $_SESSION['login_user'];

        // セッション変数を全て解除する
        $_SESSION = array();

        // セッションを切断するにはセッションクッキーも削除する。
        // Note: セッション情報だけでなくセッションを破壊する。
        if (isset($_COOKIE[session_name()])) {
            setcookie(session_name(), '', time() - 42000, '/');
        }

        // セッションを破壊する
        session_destroy();
        
        // ログ書き出し
        $log_text = array(
            $login_user->user_name,
            $login_user->display_name,
            User_Role::get_role_name($login_user->user_role),
            $_SERVER['HTTP_USER_AGENT']
        );
        Me_Logger::log(1002, $log_text);
    }

    /**
     * ページタイトル取得
     * @param boolean $is_edit
     * @return string
     */
    public function get_page_title($is_edit)
    {
        return ($is_edit) ? 'ユーザー編集' : 'ユーザー新規作成';
    }

    /**
     * パスに対するユーザーのアクセス権を確認する
     * @param string $path
     * @param string $user_id
     * @return boolean $result
     */
    public function check_access_priv($path, $user_id = null)
    {
        if ($user_id === null) {
            $login_user = $_SESSION['login_user'];
            $user_id = $login_user->_id;
        }

        // ユーザー情報は再取得する
        $user = $this->select_user($user_id);

        // 結果フラグの初期値は不許可
        $result = false;
        // ディレクトリ設定の有無確認
        if (strlen($user->path_config) > 0) {
            // 各行に分割
            $lines = Me_Util::get_lines($user->path_config);

            // パスリストに一致するかどうか
            $list_match = (boolean)(preg_match('#^(' . implode('|', $lines) . ')#', $path));
            // 初期値が不許可で許可リストに一致する場合と、初期値が許可で、不許可リストに一致しない場合は、許可
            if (($user->default_access === 'deny' && $list_match)
                || $user->default_access !== 'deny' && !$list_match) {
                $result = true;
            }
        } else if ($user->default_access !== 'deny') {
            // パスリストが無く、基本設定が許可の場合は、結果フラグを許可にする
            $result = true;
        }

        return $result;
    }
}